 function cintot(a,b,h,comment) %[coord,conn,trian,sides]=cintot(a,b,h)
%
% Synopsis: cintot(a,b,h) or cintot(a,b,h,comment)

%

% a= lower side of the cube

% b= upper side of the cube

% h = mesh size

% comment = (optional) comment to be written in the mesh file

%

% Aim: Creates a uniform 3D thetraedral mesh

% un a cube whose sides are long b-a.

% h is the step of the mesh

% Based on cintot.f by L. Formaggia (the original)

%

%         

%              5

%                o------------o 8

%              / |           /|

%             /  |          / |

%            /   |         /  |

%        6  o-------------o 7 |

%           |    |        |   |

%           |  1 o--------|---o 4

%           |   /         |  /

%           |  /          | /

%           | /           |/

%        2  o-------------o 3

%

% Copyright A. Veneziani, 2001

%

  x=[a:2*h:2*b];

  y=[a:h:b];

  z=[a:h:b];

%

  nn=(b-a)/h; %numbers of intervals for each row

  n=nn+1; %numbers of pointis for each row

  ne = nn^3; %number of cubes 

  nv = n^3; %number of points (vertexes)

  MC=[]; %Connettivity matrix

  BF=[]; %boundary faces

  BS=[]; %boundary sides

  nodi=zeros(8,1);

%

% Connectivity

%

  for ke=1:nn

   for je=1:nn

    for ie=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     aloc=conn_loc(nodi);

     MC=[MC;aloc];

    end;

   end;

  end;  

%

% Boundary treatment: I distinguish two phases just to avoid repetitions

%  Phase 1: Longitudinal boundary sides

%

%   Phase 1.1: Longitudinal sides which are the sides of the whole cube

  je=1;ke=1;

  for ie=1:nn

   BS=[BS; tre2uno(ie,je,ke,n), tre2uno(ie+1,je,ke,n)]; %(nodes 1 and 2 of each element)

  end;

  ie=nn;ke=1;

  for je=1:nn

   BS=[BS; tre2uno(ie+1,je,ke,n), tre2uno(ie+1,je+1,ke,n)]; %(nodes 2 and 3 of each element)

  end;

  je=nn;ke=1;

  for ie=nn:-1:1

   BS=[BS; tre2uno(ie+1,je+1,ke,n), tre2uno(ie,je+1,ke,n)]; %(nodes 3 and 4 of each element)

  end;

  ie=1;ke=1;

  for je=nn:-1:1

   BS=[BS; tre2uno(ie,je+1,ke,n), tre2uno(ie,je,ke,n)]; %(nodes 4 and 1 of each element)

  end;

%

  je=1;ke=nn;

  for ie=1:nn

   BS=[BS; tre2uno(ie,je,ke+1,n), tre2uno(ie+1,je,ke+1,n)]; %(nodes 5 and 6 of each element)

  end;

  ie=nn;ke=nn;

  for je=1:nn

   BS=[BS; tre2uno(ie+1,je,ke+1,n), tre2uno(ie+1,je+1,ke+1,n)]; %(nodes 6 and 7 of each element)

  end;

  je=nn;ke=nn;

  for ie=nn:-1:1

   BS=[BS; tre2uno(ie+1,je+1,ke+1,n), tre2uno(ie,je+1,ke+1,n)]; %(nodes 7 and 8 of each element)

  end;

  ie=1;ke=nn;

  for je=nn:-1:1

   BS=[BS; tre2uno(ie,je+1,ke+1,n), tre2uno(ie,je,ke+1,n)]; %(nodes 8 and 1 of each element)

  end;

%

  je=1;je=1;

  for ke=1:nn

   BS=[BS; tre2uno(ie,je,ke,n), tre2uno(ie,je,ke+1,n)]; %(nodes 1 and 5 of each element)

  end;

  ie=nn;je=1;

  for ke=1:nn

   BS=[BS; tre2uno(ie+1,je,ke,n), tre2uno(ie+1,je,ke+1,n)]; %(nodes 2 and 6 of each element)

  end;

  je=nn;ie=nn;

  for ke=1:nn;

   BS=[BS; tre2uno(ie+1,je+1,ke,n), tre2uno(ie+1,je+1,ke+1,n)]; %(nodes 3 and 7 of each element)

  end;

  ie=1;je=nn;

  for ke=1:nn;

   BS=[BS; tre2uno(ie,je+1,ke,n), tre2uno(ie,je+1,ke+1,n)]; %(nodes 4 and 8 of each element)

  end;

%

%   Phase 1.2: Longitudinal inner sides

%    Face Down

  k=1;

  for j=2:n-1

   for i=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i+1,j,k,n)]; 

   end;

  end;

  for i=2:n-1

   for j=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j+1,k,n)]; 

   end;

  end;

%

%   Face Up

  k=n;

  for j=2:n-1

   for i=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i+1,j,k,n)]; 

   end;

  end;

  for i=2:n-1

   for j=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j+1,k,n)]; 

   end;

  end;

%

%  Face Left 

  j=1;

  for k=2:n-1

   for i=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i+1,j,k,n)]; 

   end;

  end;

  for i=2:n-1

   for k=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j,k+1,n)]; 

   end;

  end;

%

%  Face Right

  j=n;

  for k=2:n-1

   for i=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i+1,j,k,n)]; 

   end;

  end;

  for i=2:n-1

   for k=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j,k+1,n)]; 

   end;

  end;

%

% Face Back

  i=1;

  for k=2:n-1

   for j=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j+1,k,n)]; 

   end;

  end;

  for j=2:n-1

   for k=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j,k+1,n)]; 

   end;

  end;

%

% Face Front

  i=n;

  for k=2:n-1

   for j=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j+1,k,n)]; 

   end;

  end;

  for j=2:n-1

   for k=1:n-1

    BS=[BS;tre2uno(i,j,k,n),tre2uno(i,j,k+1,n)]; 

   end;

  end;

%

% Boundary treatment:

%  Phase 2: Transversal boundary sides and Boundary faces

%

% 1) Down Faces : ke=1

% (Obviuosly each of the subsequent six parts could be stored in a subrotuine

%  I guess however that in this case the code is more readable...)

   ke=1;

   for je=1:nn

    for ie=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     [floc,tbloc]=b_conn_loc(nodi,'down');

     BF=[BF;floc];

     BS=[BS;tbloc]; 

    end;

   end;

% 2) Up   Faces : ke=nn

   ke=nn

   for je=1:nn

    for ie=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     [floc,tbloc]=b_conn_loc(nodi,'up');

     BF=[BF;floc];

     BS=[BS;tbloc]; 

    end;

   end;

% 3) Left Faces : je=1

   je=1;

   for ke=1:nn

    for ie=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     [floc,tbloc]=b_conn_loc(nodi,'left');

     BF=[BF;floc];

     BS=[BS;tbloc]; 

    end;

   end;

% 4) Right Faces : je=nn 

   je=nn;

   for ke=1:nn

    for ie=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     [floc,tbloc]=b_conn_loc(nodi,'right');

     BF=[BF;floc];

     BS=[BS;tbloc]; 

    end;

   end;

% 5) Back Faces : ie=1

   ie=1;

   for ke=1:nn

    for je=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     [floc,tbloc]=b_conn_loc(nodi,'back');

     BF=[BF;floc];

     BS=[BS;tbloc]; 

    end;

   end;

% 6) Front Faces : ie=nn

   ie=nn;

   for ke=1:nn

    for je=1:nn

     nodi(1)=tre2uno(ie,je,ke,n);

     nodi(2)=tre2uno(ie+1,je,ke,n);

     nodi(3)=tre2uno(ie+1,je+1,ke,n);

     nodi(4)=tre2uno(ie,je+1,ke,n);

     nodi(5)=tre2uno(ie,je,ke+1,n);

     nodi(6)=tre2uno(ie+1,je,ke+1,n);

     nodi(7)=tre2uno(ie+1,je+1,ke+1,n);

     nodi(8)=tre2uno(ie,je+1,ke+1,n);

     [floc,tbloc]=b_conn_loc(nodi,'front');

     BF=[BF;floc];

     BS=[BS;tbloc]; 

    end;

   end;

%

   MC,BS,BF

   x,y,z

   [nbs,aux1]=size(BS)

   [nbf,aux2]=size(BF)

   [noe,aux3]=size(MC)

%

% Conversion: I have to move the boundary nodes in the first positions !!!

%

  coord=zeros(nv,3);

  for k=1:n

   for j=1:n

    for i=1:n

     nold=i+(j-1)*n+(k-1)*n^2;

     shift(nold)=num_new(i,j,k,n);

     aux=shift(nold);

     coord(aux,1)=(i-1)*h;

     coord(aux,2)=(j-1)*h;

     coord(aux,3)=(k-1)*h; 

    end;

   end;

  end;

  

%

  for vu=1:nbs

   for du=1:aux1

    BS(vu,du)=shift(BS(vu,du));

   end;

  end;

  for vu=1:nbf

   for du=1:aux2

    BF(vu,du)=shift(BF(vu,du));

   end;

  end;

  for vu=1:noe

   for du=1:aux3

    MC(vu,du)=shift(MC(vu,du));

   end;

  end;

  

%

% Writing phase

%

   if (nargin<4) comment='#  \n'; 

   else  comment = ['# ', comment,'\n'];

   end;

   title=['cube_',num2str(noe),'.m++'];

   fout = fopen(title,'w');

   fprintf(fout,comment);

   fprintf(fout,'block b1\n');

   fprintf(fout,'Nodes : %i\n',nv);

   nbp=6*n^2-12*n+8;

   ity_id=1;

   ibc=10;

   for vu=1:nbp

    fprintf(fout,'%f %f %f 1 %i %i\n',coord(vu,1),coord(vu,2),coord(vu,3),ity_id,ibc);

   end;

   for vu=nbp+1:nv

    fprintf(fout,'%f %f %f 3 %i\n',coord(vu,1),coord(vu,2),coord(vu,3),ity_id);

   end;

   fprintf(fout,'Triangular Elements : %i\n',nbf);

   for vu=1:nbf

    fprintf(fout,'%i %i %i 1 %i %i\n',BF(vu,1),BF(vu,2),BF(vu,3),ity_id,ibc);

   end;

   fprintf(fout,'Boundary Sides : %i\n',nbs)

   for vu=1:nbs

    fprintf(fout,'%i %i 1 %i %i\n',BS(vu,1),BS(vu,2),ity_id,ibc);

   end;

   fprintf(fout,'Tetrahedral Elements : %i\n',noe);

   for vu=1:noe

    fprintf(fout,'%i %i %i %i\n',MC(vu,1),MC(vu,2),MC(vu,3),MC(vu,4));

   end;

   fclose(fout); 

   BS

   pause

   BF

   pause

   MC

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 function nb=tre2uno(i,j,k,n)

%

% Conversion from three coordinates to one

%

  nb=i+(j-1)*n+(k-1)*n*n;

%

 return; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 function a_loc=conn_loc(nodi)

%

% Local connectivity

%

  a_loc=zeros(6,4);

  a_loc(1,:)=[nodi(3),nodi(8),nodi(6),nodi(7)];  

  a_loc(2,:)=[nodi(1),nodi(2),nodi(4),nodi(5)];

  a_loc(3,:)=[nodi(2),nodi(3),nodi(4),nodi(6)];

  a_loc(4,:)=[nodi(6),nodi(4),nodi(5),nodi(8)];

  a_loc(5,:)=[nodi(3),nodi(6),nodi(8),nodi(4)];

  a_loc(6,:)=[nodi(6),nodi(5),nodi(4),nodi(2)];

 return;

%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 function [f_loc,tb_loc]=b_conn_loc(nodi,where)

%

% Local connectivity of the boundary

% 

%   f_loc= local face; tb_loc  = local transversal boundary

%   where=flag of the boundary side

%

  f_loc=zeros(2,3);

  tb_loc=zeros(1,2);

  switch where

   case 'left'

    f_loc(1,:)= [nodi(1),nodi(2),nodi(5)];

    f_loc(2,:)= [nodi(2),nodi(6),nodi(5)];

    tb_loc = [nodi(2),nodi(5)];

   case 'right'

    f_loc(1,:)= [nodi(7),nodi(3),nodi(8)];

    f_loc(2,:)= [nodi(3),nodi(4),nodi(8)];

    tb_loc = [nodi(3),nodi(8)];

   case 'up'

    f_loc(1,:)= [nodi(7),nodi(8),nodi(6)];

    f_loc(2,:)= [nodi(5),nodi(6),nodi(8)];

    tb_loc = [nodi(8),nodi(6)];

   case 'down'

    f_loc(1,:)= [nodi(3),nodi(2),nodi(4)];

    f_loc(2,:)= [nodi(1),nodi(4),nodi(2)];

    tb_loc = [nodi(2),nodi(4)];

   case 'front'

    f_loc(1,:)= [nodi(7),nodi(6),nodi(3)];

    f_loc(2,:)= [nodi(2),nodi(3),nodi(6)];

    tb_loc = [nodi(6),nodi(3)];

   case 'back'

    f_loc(1,:)= [nodi(4),nodi(5),nodi(8)];

    f_loc(2,:)= [nodi(1),nodi(5),nodi(4)];

    tb_loc = [nodi(4),nodi(5)];

   otherwise

    disp('Unknown flag');

  end;

 return;

%

%

 function nnew = num_new(i,j,k,n)

%

% Moving of bnodes in the first positions:

% First: nodes on the DOWN face

% Second: nodes on the UP Face

% Third: nodes on the LEFT face (except the previous already included)

% Fourth: nodes on the RIGHT face ( " " )

% Fifth: nodes on the BACK face ( " " )

% Sixth: nodes on the FRONT Face ( " " )

% Then the inner nodes

%

  if (k==1) nnew=i+n*(j-1);

  elseif (k==n) nnew=i+n*(j-1)+n^2;

  elseif (j==1) nnew=i+(n)*(k-2)+2*n^2;

  elseif (j==n) nnew=i+(n)*(k-2)+3*n^2-2*n;

  elseif (i==1) nnew=j-1+(n-2)*(k-2)+4*n^2-4*n;

  elseif (i==n) nnew=j-1+(n-2)*(k-2)+5*n^2-8*n+4;

  else nnew=(i-1)+(n-2)*(j-2)+(n-2)^2*(k-2)+6*n^2-12*n+8;

  end;

 return;

